﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.ALPS {
  [Item("LayerUniformSubScopesProcessor", "")]
  [StorableClass]
  public class LayerUniformSubScopesProcessor : UniformSubScopesProcessor {

    private ILookupParameter<IntArray> PopulationSizeParameter {
      get { return (ILookupParameter<IntArray>)Parameters["PopulationSize"]; }
    }

    [StorableConstructor]
    private LayerUniformSubScopesProcessor(bool deserializing) : base(deserializing) { }
    private LayerUniformSubScopesProcessor(LayerUniformSubScopesProcessor original, Cloner cloner)
      : base(original, cloner) {
    }
    public LayerUniformSubScopesProcessor()
      : base() {
      Parameters.Add(new LookupParameter<IntArray>("PopulationSize"));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new LayerUniformSubScopesProcessor(this, cloner);
    }

    public override IOperation Apply() {
      OperationCollection next = new OperationCollection(Successor != null ? ExecutionContext.CreateOperation(Successor) : null); // base.base.Apply not possible
      if (Operator != null) {
        List<IScope> scopes = GetScopesOnLevel(ExecutionContext.Scope, Depth.Value).ToList();
        OperationCollection inner = new OperationCollection();
        inner.Parallel = Parallel == null ? false : Parallel.Value;
        for (int i = 0; i < scopes.Count; i++) {
          var layerParameters = new LayerIntermediateParameter {
            new ValueParameter<IntValue>(PopulationSizeParameter.ActualName, new IntValue(PopulationSizeParameter.ActualValue[Math.Min(i, PopulationSizeParameter.ActualValue.Length - 1)]))
          };
          var immediateContext = new ExecutionContext(ExecutionContext.Parent, layerParameters, scopes[i]);
          inner.Add(immediateContext.CreateChildOperation(Operator, scopes[i]));
        }
        next.Insert(0, inner);
      }
      return next;
    }
  }

  [Item("LayerIntermediateParameter", "")]
  [StorableClass]
  public class LayerIntermediateParameter : ParameterizedNamedItem, IEnumerable<IParameter> {
    [StorableConstructor]
    private LayerIntermediateParameter(bool deserializing) : base(deserializing) { }
    private LayerIntermediateParameter(LayerIntermediateParameter original, Cloner cloner)
      : base(original, cloner) {
    }
    public LayerIntermediateParameter()
      : base("LayerIntermediateParameter") {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new LayerIntermediateParameter(this, cloner);
    }

    public void Add(IParameter parameter) {
      Parameters.Add(parameter);
    }

    public IEnumerator<IParameter> GetEnumerator() {
      return Parameters.GetEnumerator();
    }
    IEnumerator IEnumerable.GetEnumerator() {
      return GetEnumerator();
    }
  }
}